#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/wb.h>
#include <proto/intuition.h>

struct Screen *screen = NULL;
char *filer_binary = NULL;
static struct Task *Caller = NULL;
BOOL disable_flash_beep = FALSE;

struct Window * APICALL (*Original_OpenWindowTagList)(struct IntuitionIFace *Self, const struct NewWindow * newWindow, const struct TagItem * tagList);
VOID * APICALL (*Original_DisplayBeep)(struct IntuitionIFace *Self, struct Screen *screen);


VOID * APICALL Patched_DisplayBeep(struct IntuitionIFace *Self, struct Screen *screen)
{

	if (disable_flash_beep == TRUE) {
		disable_flash_beep = FALSE;
		return NULL;
	} else {
		Original_DisplayBeep(Self,screen);
	}
	
}


struct Window * APICALL Patched_OpenWindowTagList(struct IntuitionIFace *Self, const struct NewWindow * nw, const struct TagItem * tagList) 
{

	Caller = (struct Task *)IExec->FindTask(NULL);

	// checking if this is WB window which we need to patch	
	if (nw && !stricmp(Caller->tc_Node.ln_Name, "Workbench") && (nw->Flags & WFLG_WBENCHWINDOW) && !(nw->Flags & WFLG_BACKDROP))
	{

		char fullpath[1024];
		ULONG type;
		
		IWorkbench->WhichWorkbenchObject(NULL,screen->MouseX,screen->MouseY,
							WBOBJA_Type,&type,
							WBOBJA_FullPath,&fullpath,
							WBOBJA_FullPathSize,1024,
							TAG_DONE);			
		
		// we hijack only WBDISK types, so commands like "wbrun" and Filer's "Workbench Folder" works as before		
		if (type != WBDISK) {
			return Original_OpenWindowTagList(Self,nw, tagList); 
		};

		// set to disable flashing of DisplayBeep() only for our hack, and keep it original for other processe
		disable_flash_beep = TRUE;
		
		// now construct a string to run: name of binary and partition in quotes.
		// (quotes are must for taking spaces into account, like for "Ram Disk:")		
		char *ready_string = NULL;							
		asprintf(&ready_string, "%s%c%c%s%c", filer_binary, ' ', '"', fullpath, '"');

		IDOS->SystemTags(ready_string,
			SYS_Asynch,TRUE,
			SYS_Input,IDOS->DupFileHandle(IDOS->Input()),
			SYS_Output,IDOS->DupFileHandle(IDOS->Output()),
			TAG_DONE);
							
		return NULL;		
		
	} else {
		
		return Original_OpenWindowTagList(Self,nw, tagList);
	}

}


void init()
{
	screen = IIntuition->LockPubScreen(NULL);
	
	Original_OpenWindowTagList = IExec->SetMethod((struct Interface *)IIntuition, offsetof(struct IntuitionIFace, OpenWindowTagList),  (APTR)Patched_OpenWindowTagList);
	Original_DisplayBeep = IExec->SetMethod((struct Interface *)IIntuition, offsetof(struct IntuitionIFace, DisplayBeep),  (APTR)Patched_DisplayBeep);
}

void deinit()
{
	if(Original_DisplayBeep) {
		IExec->SetMethod((struct Interface *)IIntuition, offsetof(struct IntuitionIFace, DisplayBeep),  (APTR)Original_DisplayBeep);
	}

	if(Original_OpenWindowTagList) {
		IExec->SetMethod((struct Interface *)IIntuition, offsetof(struct IntuitionIFace, OpenWindowTagList),  (APTR)Original_OpenWindowTagList);
	}

	if(screen) IIntuition->UnlockPubScreen(NULL, screen);

}


int main (int argc, char* argv[])
{	
	
	uint32 sigmask = 0, signals;

	if (argc !=2) {
		printf ("you need to specify a PATH to the Filer binary\n");
		exit(0);
	}

	filer_binary = argv[1];
   
	init();
	IDOS->Printf("patching activated\n");
	
	// CTRL+C check
	while(1) {
		signals = IExec->Wait(sigmask|SIGBREAKF_CTRL_C);
	
		if(signals & SIGBREAKF_CTRL_C) {
			IDOS->Printf("patching cleaned\n");
			break;
		}
	}	
	
	deinit();
}
